/*
   A Tree is defined by the following grammar.

     Tree ::= '(' String Tree+ ')'
            | String

   where '(' and ')' are literals, and Tree+ means "one or more".

   This language also assumes that all tokens have white space around them.

   The static method getTree() is essentially a recursive descent
   parser for this Tree langauge.
*/

public class BuildTree
{
   public static Tree buildTree(String expression) throws ParseException
   {
      Tokenizer tokens = new Tokenizer(expression);  // Tokenizer is defined
                                                     // in Tokenizer.java
      Tree result = getTree(tokens); // parse the tokens

      if ( tokens.hasToken() )  // there shouldn't be any more tokens
         throw new ParseException("syntax error: unexpected input: "+tokens.peekToken()+"\n"+tokens);

      return result;
   }//buildTree()


   private static Tree getTree(Tokenizer tokens) throws ParseException
   {
      Tree result;
      String token = "";

      if ( tokens.hasToken() )
      {
         token = tokens.nextToken(); // consume one token
      }
      else
      {
         throw new ParseException("syntax error: unexpected end of input"+"\n"+tokens);
      }

      if ( token.equals("(") )           // look for a parenthesized tree
      {
         if ( tokens.hasToken() )
         {
            result = new Tree( tokens.nextToken() ); // consume the root of the tree
         }
         else
         {
            throw new ParseException("syntax error: unexpected end of input"+"\n"+tokens);
         }

         // consume first sub tree
         result.addSubTree( getTree(tokens) );

         // one character look ahead
         if ( tokens.hasToken() )
         {
            token = tokens.peekToken();
         }
         else
         {
            throw new ParseException("syntax error: unexpected end of input"+"\n"+tokens);
         }

         // consume any other sub trees
         while ( !token.equals(")") )
         {  // consume another sub tree
            result.addSubTree( getTree(tokens) );
             // one character look ahead
            if ( tokens.hasToken() )
            {
               token = tokens.peekToken();
            }
            else
            {
               throw new ParseException("syntax error: unexpected end of input"+"\n"+tokens);
            }
         }
         tokens.match(")");  // consume the matching ")"
      }
      else
      {
         result = new Tree(token);       // the tree must be just the root
      }

      return result;
   }//getTree()

}//BuildTree
