/*
  This program tests your implentations of Evaluate.java,
  and AST2infix.java. When your implentations are correct,
  the output from this program should look exactly like
  the contents of the file output.txt.

  You should consider creating your own test expressions to make
  sure your implementations are correct.
*/

public class Tests
{
   public static void main(String[] args)
   {
      String[] exprs =
      {
         "( prog ( if true 0 1 ) )",

         "( prog ( if false 0 1 ) )",

         "( prog " +
           "( begin " +
             "( var b true ) " +
             "( if b true flase ) ) )",

         "( prog " +
           "( begin " +
             "( var i 5 ) " +
             "( while ( > i 0 ) " +
               "( begin " +
                 "( print i ) " +
                 "( set i ( - i 1 ) ) ) ) ) )",

         "( prog " +
           "( begin " +
            "( var n 1 ) " +
            "( var sum 0 ) " +
            "( while ( < n 5 ) " +
              "( begin " +
                "( set sum ( + sum n ) ) " +
                "( set n ( + n 1 ) ) " +
                "( print n ) " +
                "( print sum ) ) ) " +
            "sum ) )",

        "( prog " +
          "( define g ( lambda ( + 1 2 3 4 5 6 ) ) ) " +
          "( apply  g ) )",  // g is a function that has no paramerters

        "( prog " +
          "( define f ( lambda x y z ( + x y z ) ) ) " +
          "( apply  f 5 6 7 ) )",

        "( prog ( apply ( lambda x ( * x x ) ) 5 ) )",

        "( prog ( apply ( lambda  ( + 2 2 ) )  ) )",  // this anonymous function
                                                      // has no parameters
        "( prog ( apply ( lambda u v ( + u v ) ) 3 4 ) )",

         "( prog " +
           "( define x 20 ) " +  // global x
           "( begin " +
             "( var x 10 ) " +   // local x
             "( apply ( lambda z ( + z x ) ) 5 ) ) )"  // non local reference
      };

      for (int i = 0; i < exprs.length; i++)
      {
         try
         {
            Tree ast = BuildTree.buildTree( exprs[i] );

            System.out.println( PrettyPrinter.prettyPrint( ast ) + "\n" );
            System.out.println( AST2infix.ast2infix( ast ) + "\n" );

            Value value = Evaluate.evaluate( ast );
            System.out.println( "result = " + value );
         }
         catch (ParseException e)
         {
            System.out.println(e);
         }
         catch (EvalException e)
         {
            System.out.println(e);
         }
         System.out.println("=============================================");
      }
   }//main

}//Tests
